import java.io.*;
import java.util.*;

/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 27.09.16
 */

public class TestGen {
    private final boolean FOR_POLYGON = true;

    public static final int MAX_N = 1000;
    public static final int MAX_VAL = 1_000_000_000;
    public static final int MAX_VAL2 = 1_000_000;

    private void shuffle(List<?> a) {
        Collections.shuffle(a, random);
    }

    int testCount;
    static Random random = new Random(0xCEEECCEE);

    private String folder;
    boolean isSamples = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void println(PrintWriter writer, int... arr) {
        for (int i = 0; i < arr.length; ++i)
            if (i + 1 == arr.length) writer.println(arr[i]);
            else writer.print(arr[i] + " ");
    }

    public void print(int[][] d, String descr) throws IOException {
        String path = (FOR_POLYGON ?
                String.format("%d",++testCount) :
                String.format("../%s/%02d", folder, ++testCount));

        PrintWriter out = new PrintWriter(path);
        System.out.println("Generating test #" + testCount + ": " + descr);
        int n = d.length;
        out.println(n);
        for (int i = 0; i < n; ++i) {
            println(out, d[i]);
        }
        out.close();
    }

    public void genSamples() throws IOException {
        setFolder("tests");
        isSamples = true;

        System.out.println("==== Generating samples ===");
        print(new int[][]{
                {3, 2, 1, 0},
                {2, 1, 0, 1},
                {1, 0, 1, 2},
                {0, 1, 2, 3}}, "sample1, answer yes");
        print(new int[][]{
                {1, 0, 2},
                {1, 1, 1},
                {1, 1, 0}
        }, "sample2, answer no");
        print(new int[][]{
                {7, 0, 2, 1},
                {9, 2, 0, 3},
                {0, 7, 9, 6},
                {6, 1, 3, 0}
        }, "sample3, answer yes");
        isSamples = false;
    }

    void genHandleTests() throws IOException {
        print(new int[][]{{0}}, "one point");
        print(new int[][]{{MAX_VAL}}, "one point");
    }


    public void genTests() throws IOException {
        setFolder("tests");
        int n = 500;
        print(allZeroes(10), "all zeroes, n = " + n);
        print(allEq(10), "answer NO, all numbers are equals");
        print(allEqExcludeDiag(20), "answer NO, all equals, exclude main diagonal");
        print(oneMax(20), "answer NO, one max");
        print(oneMax(n), "answer NO, one max");
        print(eqParts(n), "answer YES, eqParts, n = " + n);
        print(eqParts(n), "answer YES, eqParts, n = " + n);
        print(ansYes(n), "answer YES, n = " + n);
        print(ansYes(n), "answer YES, n = " + n);
        print(randNo(n), "answer NO, n = " + n);
        print(randNo(n), "answer NO, n = " + n);
        print(randNo(n, 1), "answer NO, n = " + n);
        print(randNo(n, 3), "answer NO, n = " + n);
        print(ansYes(n, 2), "2 groups of points, n = " + n);
        print(ansYes(n, 3), "3 groups of points, n = " + n);

        n = 1000;
        print(allZeroes(n), "all zeroes, n = " + n);
        print(allEq(n), "answer NO, all numbers are equals, n = " + n);
        print(ansYes(n, 1.0), "answer YES, n = " + n);
        print(randNo(n), "answer NO, n = " + n);
        print(randNo(n, 2), "answer NO, n = " + n);
        print(randNo(n, 1.0), "answer NO, n = " + n);
        print(ansYes(n, 30), "30 groups of points, n = " + n);

        print(fullRandom(n), "full random");
    }

    private int[][] eqParts(int n) {
        n = (n / 3) * 3;
        int mn = randInt(0, MAX_VAL2 / 2);
        int mx = randInt(MAX_VAL2 / 2 + 1, MAX_VAL2);
        int[] p1 = genRandPerm(n);
        int[] p2 = genRandPerm(n);
        int[] x = new int[n];
        for (int i = 0; i < n / 3; ++i) {
            x[i] = 0;
        }
        for (int i = n / 3; i < 2 * (n / 3); ++i) {
            x[i] = randInt(mn, (mx + mn) / 2);
        }
        for (int i = 2 * (n / 3); i < n; ++i) {
            x[i] = mx;
        }
        Arrays.sort(x);
        return calcD(x, p1, p2);
    }

    private int[][] fullRandom(int n) {
        int[][] d = new int[n][n];
        for (int i = 0; i < n; ++i)
            for (int j = 0; j < n; ++j)
                d[i][j] = randInt(0, MAX_VAL2);
        return d;
    }

    private int[][] randNo(int n) {
        return randNo(n, DEFAULT_PROB);
    }

    private int[][] randNo(int n, int numGroups) {
        return randNo(n, DEFAULT_PROB, numGroups);
    }

    private int[][] randNo(int n, double prob) {
        return randNo(n, prob, n);
    }

    private int[][] randNo(int n, double prob, int numGroups) {
        int[][] d = ansYes(n, prob, numGroups);
        int i = randInt(0, n - 1);
        int j = randInt(0, n - 1);
        d[i][j] = randInt(0, MAX_VAL);
        return d;
    }

    final double DEFAULT_PROB = 0.1;
    private int[][] ansYes(int n) {
        return ansYes(n, DEFAULT_PROB, n);
    }

    private int[][] ansYes(int n, int numGroups) {

        return ansYes(n, DEFAULT_PROB, numGroups);
    }

    private int[][] ansYes(int n, double prob) {
        return ansYes(n, prob, n);
    }

    private int[][] ansYes(int n, double prob, int numGroups) {
        int[] part = genPartition(n, numGroups);
        int[] groupVal = new int[n];
        int t = 0, ptr = 0;
        int[] a = new int[n];
        for (int i = 0; i < numGroups; ++i) {
            if ((t + part[i]) * 1.0 / n < prob) {
                t += part[i];
                groupVal[i] = randInt(MAX_VAL2 + 1, MAX_VAL);
            } else {
                groupVal[i] = randInt(0, MAX_VAL);
            }
            for (int j = 1; j <= part[i]; ++j) {
                a[ptr++] = groupVal[i];
            }
        }
        Arrays.sort(a);
        int[] p1 = genRandPerm(n);
        int[] p2 = genRandPerm(n);
        return calcD(a, p1, p2);
    }

    private int[][] allZeroes(int n) {
        return new int[n][n];
    }

    private int[][] allEq(int n) {
        int x = randInt(1, MAX_VAL2);
        int[][] d = new int[n][n];
        for (int i = 0; i < n; ++i)
            for (int j = 0; j < n; ++j) {
                d[i][j] = x;
            }
        return d;
    }

    private int[][] allEqExcludeDiag(int n) {
        int[][] d = allEq(n);
        for (int i = 0; i < n; ++i)
            d[i][i] = 0;
        return d;
    }

    private int[][] calcD(int[] x, int[] p1, int[] p2) {
        int n = x.length;
        for (int i = 0; i + 1 < n; ++i)
            if (x[i] > x[i + 1]) {
                throw new RuntimeException("x must to be nondecreasing");
            }
        int[][] d = new int[n][n];
        for (int i = 0; i < n; ++i)
            for (int j = 0; j < n; ++j) {
                d[i][j] = Math.abs(x[p1[i]] - x[p2[j]]);
            }
        return d;
    }

    private int[][] oneMax(int n) {
        int[][] d = ansYes(n);
        int mx = -1, mxi = -1, mxj = -1;
        for (int i = 0; i < n; ++i)
            for (int j = 0; j < n; ++j)
                if (d[i][j] > mx) {
                    mx = d[i][j];
                    mxi = i;
                    mxj = j;
                }
        for (int i = 0; i < n; ++i)
            for (int j = 0; j < n; ++j) {
                if (d[i][j] == mx && (i != mxi || j != mxj)) {
                    d[i][j]--;
                }
            }
        return d;
    }

    private int maxLog(int n) {
        int pw = 1;
        while (2 * pw <= n) pw *= 2;
        return pw;
    }

    private int randMaxVal() {
        return randInt(-MAX_VAL, MAX_VAL);
    }

    private void shuffle(int[] a, int from, int to) {
        int[] b = Arrays.copyOfRange(a, from, to);
        Integer[] bInteger = new Integer[b.length];
        for (int i = 0; i < b.length; ++i)
            bInteger[i] = b[i];

        List<Integer> arr = Arrays.asList(bInteger);
        shuffle(arr);
        for (int i = from; i < to; ++i)
            a[i] = arr.get(i - from);
    }

    private int[] toIntArray(Integer[] a) {
        int[] ret = new int[a.length];
        for (int i = 0; i < ret.length; ++i)
            ret[i] = a[i];
        return ret;
    }


    private static int randInt(int l, int r) {
        return random.nextInt(r - l + 1) + l;
    }

    private static int randInt(int n) {
        return random.nextInt(n) + 1;
    }

    public void run() throws IOException {
        if (!FOR_POLYGON) {
            new File("../tests").mkdir();
        }

        genSamples();
        genHandleTests();
        genTests();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }

    private int[] genCombine(int n, int k) {
        if (k > n)
            throw new IllegalArgumentException("k > n");

        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            p.add(i + 1);
        shuffle(p);
        int[] ret = new int[k];
        List<Integer> sublist = p.subList(0, k);
        for (int i = 0; i < k; ++i)
            ret[i] = sublist.get(i);
        Arrays.sort(ret);
        return ret;
    }

    private int[] genPartitionViaCombine(int n, int[] comb) {
        int k = comb.length + 1;
        int[] ret = new int[k];
        if (k >= 2)
            ret[0] = comb[0];

        for (int i = 1; i + 1 < k; ++i)
            ret[i] = comb[i] - comb[i - 1];
        ret[k - 1] = k >= 2 ?  n - comb[k - 2] : n;
        return ret;
    }

    public int[] genPartition(int sum, int k) {
        int[] comb = genCombine(sum - 1, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genPartitionWithZeroes(int sum, int k) {
        int[] comb = genCombineWithEq(sum, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genCombineWithEq(int n, int k) {
        int[] ret = new int[k];
        for (int i = 0; i < k; ++i)
            ret[i] = randInt(0, n);
        Arrays.sort(ret);
        return ret;
    }

    private int[] genRandPerm(int n) {
        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i) {
            p.add(i);
        }
        Collections.shuffle(p, random);
        int[] ret = new int[n];
        for (int i = 0; i < n; ++i) {
            ret[i] = p.get(i);
        }
        return ret;
    }
}

