import java.io.*;
import java.util.*;

/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 27.09.16
 */

public class TestGen {
    private final boolean FOR_POLYGON = true;

    private void shuffle(List<?> a) {
        Collections.shuffle(a, random);
    }

    int testCount;
    static Random random = new Random(0xCEEECCEE);

    public static final int MAX_N = 1000_00;//10^5
    public static final int MAX_VAL = 1_000_000_000;

    private String folder;
    boolean isSamples = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void println(PrintWriter writer, int... arr) {
        for (int i = 0; i < arr.length; ++i)
            if (i + 1 == arr.length) writer.println(arr[i]);
            else writer.print(arr[i] + " ");
    }

    public void print(int[] a, String descr) throws IOException {
        String path = (FOR_POLYGON ?
                String.format("%d",++testCount) :
                String.format("../%s/%02d", folder, ++testCount));

        PrintWriter out = new PrintWriter(path);
        System.out.println("Generating test #" + testCount + ": " + descr);
        out.println(a.length);
        println(out, a);
        out.close();
    }

    public void genSamples() throws IOException {
        setFolder("tests");
        isSamples = true;

        System.out.println("==== Generating samples ===");

        print(new int[]{1, 2, 3}, "sample1");

        print(new int[]{2, 3, 2}, "sample2");
        isSamples = false;
    }

    void genHandleTests() throws IOException {
        print(new int[]{1}, "n = 1");
        print(new int[]{2}, "n = 1");
        print(new int[]{3}, "n = 1");
        print(new int[]{MAX_VAL}, "n=1");
        print(new int[]{5, 2, 3}, "a[0] 3 % = 2, n = 3");
        print(new int[]{5, 2}, "a[0] % 3 = 2, a[1] % 2 = 0, n = 2");
        print(new int[]{5, 3}, "a[0] % 3 = 2, a[1] % 2 = 1, n = 2");
        print(new int[]{12, 1}, "a[0] % 3 = 0, n = 2");
        print(new int[]{10, 5, 10, 20, 23}, "a[0] % 3 = 1, large result");
        print(new int[]{10, 5}, "a[0] % 3 = 1, a[1] % 2 = 1");
        print(new int[]{13, 6}, "a[0] % 3 = 1, a[1] % 2 = 0");
    }

    public void genBigTests() throws IOException {
        setFolder("tests");
        int n = 1000;
        int[] a = new int[n];
        for (int t = 0; t < 4; ++t) {
            for (int i = 0; i < n; ++i) {
                a[i] = randInt(1, MAX_VAL);
            }
            while (a[0] > 1 && a[0] % 3 != t % 3) --a[0];
            if (t % 3 == 2 && a[1] > 1) {
                a[1] -= (a[1] + 1) % 2;
            }
            print(a, "random test, n = 1000");
        }

        n = MAX_N;
        a = new int[n];
        for (int t = 0; t < 3; ++t) {
            for (int i = 0; i < n; ++i) {
                a[i] = randInt(1, MAX_VAL);
            }

            while (a[0] > 1 && a[0] % 3 != t % 3) --a[0];
            if (t % 3 == 2 && a[1] > 1) {
                a[1] -= (a[1] + 1) % 2;
            }
            print(a, "random test, n = " + n);
        }

        for (int i = 0; i < n; ++i) {
            a[i] = MAX_VAL;
        }
        print(a, "a[i] = 10^9, n = " + n);
        a[0]--;
        print(a, "a[i] = 10^9, n = " + n);
        a[0]--;
        a[1]--;
        print(a, "a[i] = 10^9, n = " + n);
    }

    private int maxLog(int n) {
        int pw = 1;
        while (2 * pw <= n) pw *= 2;
        return pw;
    }

    private int randMaxVal() {
        return randInt(-MAX_VAL, MAX_VAL);
    }

    private void shuffle(int[] a, int from, int to) {
        int[] b = Arrays.copyOfRange(a, from, to);
        Integer[] bInteger = new Integer[b.length];
        for (int i = 0; i < b.length; ++i)
            bInteger[i] = b[i];

        List<Integer> arr = Arrays.asList(bInteger);
        shuffle(arr);
        for (int i = from; i < to; ++i)
            a[i] = arr.get(i - from);
    }

    private int[] toIntArray(Integer[] a) {
        int[] ret = new int[a.length];
        for (int i = 0; i < ret.length; ++i)
            ret[i] = a[i];
        return ret;
    }


    private static int randInt(int l, int r) {
        return random.nextInt(r - l + 1) + l;
    }

    private static int randInt(int n) {
        return random.nextInt(n) + 1;
    }

    public void run() throws IOException {
        if (!FOR_POLYGON) {
            new File("../tests").mkdir();
        }

        genSamples();
        genHandleTests();
        genBigTests();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }

    private int[] genCombine(int n, int k) {
        if (k > n)
            throw new IllegalArgumentException("k > n");

        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            p.add(i + 1);
        shuffle(p);
        int[] ret = new int[k];
        List<Integer> sublist = p.subList(0, k);
        for (int i = 0; i < k; ++i)
            ret[i] = sublist.get(i);
        Arrays.sort(ret);
        return ret;
    }

    private int[] genPartitionViaCombine(int n, int[] comb) {
        int k = comb.length + 1;
        int[] ret = new int[k];
        if (k >= 2)
            ret[0] = comb[0];

        for (int i = 1; i + 1 < k; ++i)
            ret[i] = comb[i] - comb[i - 1];
        ret[k - 1] = k >= 2 ?  n - comb[k - 2] : n;
        return ret;
    }

    public int[] genPartition(int sum, int k) {
        int[] comb = genCombine(sum - 1, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genPartitionWithZeroes(int sum, int k) {
        int[] comb = genCombineWithEq(sum, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genCombineWithEq(int n, int k) {
        int[] ret = new int[k];
        for (int i = 0; i < k; ++i)
            ret[i] = randInt(0, n);
        Arrays.sort(ret);
        return ret;
    }
}

