import java.io.*;
import java.util.Arrays;
import java.util.StringTokenizer;

/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 27.09.16
 */
public class distances_ip implements Closeable {

    public static final int N = 100000;
    public static final int MAX_VAL = 1_000_000_000;

    class Pair implements Comparable<Pair> {
        int val, index;

        Pair(int val, int index) {
            this.val = val;
            this.index = index;
        }

        @Override
        public int compareTo(Pair o) {
            return val - o.val;
        }
    }

    FastScanner in;
    PrintWriter out;

    public distances_ip() throws IOException {
        in = new FastScanner(new File("distances.in"));
        out = new PrintWriter("distances.out");
    }

    @Override
    public void close() throws IOException {
        out.close();
    }

    public void run() {
        int n = in.nextInt();
        int[][] a = new int[n][n];
        int mx = -1, mxi = 0;
        for (int i = 0; i < n; ++i)
            for (int j = 0; j < n; ++j) {
                a[i][j] = in.nextInt();
                if (a[i][j] > mx) {
                    mx = a[i][j];
                    mxi = i;
                }
            }

        int mxj = 0;
        for (int j = 0; j < n; ++j)
            if (a[mxi][j] == 0) {
                mxj = j;
            }
        if (mxj == -1) {
            out.println("NO");
            return;
        }
        Pair[] row = new Pair[n];
        Pair[] col = new Pair[n];

        for (int j = 0; j < n; ++j) {
            row[j] = new Pair(a[mxi][j], j);
        }
        for (int i = 0; i < n; ++i) {
            col[i] = new Pair(a[i][mxj], i);
        }
        int[] p1 = new int[n];
        int[] p2 = new int[n];
        int[] xs = new int[n];
        getPerm(row, xs, p2);
        getPerm(col, xs, p1);

        //Check
        boolean all = true;
        for (int i = 0; i < n; ++i)
            for (int j = 0; j < n; ++j) {
                all &= Math.abs(xs[p1[i]] - xs[p2[j]]) == a[i][j];
            }
        if (!all) {
            out.println("NO");
            return;
        }
        out.println("YES");
        printArray(xs, out);
        printArray(p1, out);
        printArray(p2, out);
    }

    private void printArray(int[] a, PrintWriter out) {
        int n = a.length;
        for (int i = 0; i < n; ++i)
            if (i + 1 != n) {
                out.print((a[i] + 1) + " ");
            } else {
                out.println(a[i] + 1);
            }
    }

    private Pair[] getPerm(Pair[] a, int[] ansx, int[] ansperm) {
        Arrays.sort(a);
        Pair[] p = new Pair[a.length];
        for (int i = 0; i < a.length; ++i) {
            ansperm[a[i].index] = i;
            ansx[i] = a[i].val - 1;
        }
        return p;
    }

    public static void main(String[] args) throws IOException {
        distances_ip instance = new distances_ip();
        instance.run();
        instance.close();
    }

    class FastScanner {
        BufferedReader br;
        StringTokenizer st;

        FastScanner(File f) {
            try {
                br = new BufferedReader(new FileReader(f));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }

        String next() {
            while (st == null || !st.hasMoreTokens()) {
                try {
                    st = new StringTokenizer(br.readLine());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            return st.nextToken();
        }

        int nextInt() {
            return Integer.parseInt(next());
        }
    }
}
