import ru.ifmo.testlib.Checker;
import ru.ifmo.testlib.InStream;
import ru.ifmo.testlib.Outcome;

public class Check implements Checker {
    public static final int MAX_VAL = 1_000_000_000;

    public boolean isPerm(int[] a) {
        int n = a.length;
        boolean[] use = new boolean[n];
        for (int x : a) {
            if (x < 0 || x >= n || use[x]) {
                return false;
            }
            use[x] = true;
        }
        return true;
    }

    String check(int[] x, int[] a, int[] b, int[][] d) {
        int n = x.length;
        for (int i = 0; i < n; ++i)
            if (Math.abs(x[i]) > MAX_VAL) {
                return String.format("x[%d] = %d, |%d|>=%d", i, x[i], x[i], MAX_VAL);
            }
        for (int i = 0; i < n; ++i) {
            --a[i];
            --b[i];
        }
        if (!isPerm(a)) {
            return "a isn't permutation";
        }
        if (!isPerm(b)) {
            return "b isn't permutation";
        }
        for (int i = 0; i < n; ++i)
            for (int j = 0; j < n; ++j) {
                if (d[i][j] != Math.abs(x[a[i]] - x[b[j]])) {
                    return String.format("d[%d][%d] != |x[a[%d]] - x[b[%d]]|", i, j, i, j);
                }
            }
        return null;
    }

    int[] readArray(int n, InStream is) {
        int[] ar = new int[n];
        for (int i = 0; i < n; ++i) {
            ar[i] = is.nextInt();
        }
        return ar;
    }

    @Override
    public Outcome test(InStream inf, InStream ouf, InStream ans) {
        String jr = ans.nextToken().toLowerCase();
        String prt = ouf.nextToken().toLowerCase();

        if (!jr.equals("yes") && !jr.equals("no"))
            return new Outcome(Outcome.Type.FAIL, "Expected YES or NO from Jury, but received " + prt);

        if (!prt.equals("yes") && !prt.equals("no"))
            return new Outcome(Outcome.Type.PE, "Expected YES or NO from part, but received " + prt);

        int n = inf.nextInt();
        int[][] d = new int[n][n];
        for (int i = 0; i < n; ++i)
            for (int j = 0; j < n; ++j) {
                d[i][j] = inf.nextInt();
            }

        if (jr.equals("yes")) {
            int[] x = readArray(n, ans);
            int[] a = readArray(n, ans);
            int[] b = readArray(n, ans);
            String msg = check(x, a, b, d);
            if (msg != null) {
                return new Outcome(Outcome.Type.FAIL, msg);
            }
        }

        if (prt.equals("yes")) {
            int[] x = readArray(n, ouf);
            int[] a = readArray(n, ouf);
            int[] b = readArray(n, ouf);
            String msg = check(x, a, b, d);
            if (msg != null) {
                return new Outcome(Outcome.Type.WA, msg);
            }
        }

        if (jr.equals("yes") && prt.equals("no")) {
            return new Outcome(Outcome.Type.WA, "Jury has answer, but part hasn't!");
        } else if (jr.equals("no") && prt.equals("yes")) {
            return new Outcome(Outcome.Type.FAIL, "Jury hasn't answer, but part has correct answer!");
        } else if (jr.equals("no")) {
            ouf.seekEoF();
            ouf.close();
            return new Outcome(Outcome.Type.OK, "No solution!");
        }

        ans.nextLine();
        ans.close();
        ouf.seekEoF();
        ouf.close();
        return new Outcome(Outcome.Type.OK, "Good job!");
    }
}
